/* CD-Installer.cmd
 *
 * command line application installer for Suntan Special
 *
 * Copyright (C) 2013 Blonde Guy
 * All Rights Reserved
 */

Signal on Error
signal on Syntax
 
/* check argument string */
parse arg command
if command = '' then do
   call LogCdi "CD-Installer version 0.66 by Blonde Guy"
   call LogCdi "usage: cd-installer <application name>"
   return 100
end

call 'cd-env'

call setlocal

/* load global values */
originalDir = Directory()
appsDir = strip(cdglobal('appsDir'), 't', '\')
sourcePath = cdglobal('sourceDir')
suntanDir = strip(sysini("User", 'Suntan Special', 'Settings'), 'T', d2c(0))
if suntanDir = 'ERROR:' then do
   call LogCdi "Warning: Suntan directory is not set"
   suntanDir = '.'
end
useRSJ = cdglobal('useRSJ')
         
parse var command vcomm varg
vcomm = strip(vcomm)

/* parse source path into source directories */
j = 0
do while sourcePath \= ''
   parse var sourcePath sourceDir ';' sourcePath
   if sourceDir \= '' then do
      j = j + 1
      Dir.j = sourceDir
      if translate(left(Dir.j, 2)) = translate(useRSJ) then do
         if useRSJ <> '' & SysSearchPath('PATH', 'CDATTACH.EXE') <> '' then do
            address cmd '@cdattach' useRSJ
            call LogCdi "Mounted" useRSJ "using RSJ; return code" rc
         end
      end
   end
end
Dir.0 = j

sfa = 200
do while sfa = 200
   sfa = SearchForApp()
end

call endlocal   
return sfa

SearchForApp:

/* check that the application name is available on the source path */
do j = 1 to Dir.0
   sourceDir = Dir.j
   call Directory sourceDir
   call SysFileTree '*.*', 'Apps', 'DO'

   do i=1 to Apps.0
      if compare(translate(vcomm), translate(filespec('name',Apps.i))) = 0 then do

         /* check for a web app that is newer */
         updateDir = suntanDir || '\update'
         showWebApps = cdglobal('showWebApps')
         if showWebApps <> 0 then do
            if stream(updateDir || '\' || vcomm, 'c', 'query exists') <> '' then do
               localDate = SysGetFileDateTime(Apps.i || '\cdi.cmd')
               webDate = SysGetFileDateTime(updateDir || '\' || vcomm)
               if localdate < webDate then do
                  call WebInstall 'web', vcomm, varg
                  return 0
               end
            end
         end

         /* execute the command */
         appcount = i
         status = 'OK'
         cr = d2c(13)
         do while status = 'OK'
            cmdname = substr(Apps.appcount, lastpos('\', Apps.appcount) + 1)
            call LogCdi "!application Application:" cmdname
            retcode = ExecuteCommand(Apps.appcount, varg)
            if retcode <> 0 then do
               cmdname = substr(Apps.appcount, lastpos('\', Apps.appcount) + 1)
               status = MessageDialog("Error", 5, "The command" cmdname,
                                      "did not execute successfully." || cr || cr || "Press OK",
                                      "to repeat the command." || cr || "Press Cancel to",
                                      "skip the command.")
               if status = 'Cancel' then do
                  return retcode
               end
            end
            else do
               return retcode
            end
         end
      end
   end
end

/* look for a CD-ROM drive that has changed drive letter */
testFile = '\Setup\' || vcomm || '\cdi.cmd'
map = SysDriveMap('c', 'used')
appFound = 0
do while map <> ''
   parse var map testDrive map
   if stream(testDrive || testFile, 'c', 'query exists') <> '' then do
      map = ''
      cdLetter = cdglobal('cdLetter')
      if cdLetter <> '' then do
         do j=1 to Dir.0
            if left(Dir.j, 1) = cdLetter then do
               Dir.j = left(testDrive, 1) || substr(Dir.j, 2)
            end
         end
      end
      call LogCdi "Found" vcomm "in" testDrive || testFile
      appFound = 1
   end
end

if appFound = 0 then do
   cr = d2c(13)
   dialogText = "Product Name:" vcomm || cr || cr ||,
                "Suntan Special cannot find" vcomm || ". If you are",
                "installing from CD-ROM, insert the Suntan Special",
                "CD-ROM into the drive."
   choice1 = "Try again."
   choice2 = "Skip this command."
   rc = choice("Message", dialogText, choice1, choice1, choice2)
   if rc = choice2 then do
      call LogCdi "User chose to skip installation of" vcomm
      return 300
   end
end

/* the command was not found on the user's CD, either */
return 200

Syntax:
Error:
call LogCdi "Condition" Condition('c') "was raised"
call LogCdi "CD-Installer Error" rc "on line" sigl
call LogCdi ErrorText(rc)
return 42
